------------------------------------------------------------------
--InventoryInsight.lua
--Author: Vicster0
--v1.6.0
--[[
	Collects inventory data for all characters on a single account 
	including the shared bank and makes this information available 
	on tooltips across the entire account providing the player 
	with useful insight into their account wide inventory.
]]
--[[DISCLAIMER
	This Add-on is not created by, affiliated with or sponsored by 
	ZeniMax Media Inc. or its affiliates. The Elder Scrolls® and 
	related logos are registered trademarks or trademarks of 
	ZeniMax Media Inc. in the United States and/or other countries.
	All rights reserved."
]]
------------------------------------------------------------------

InventoryInsight = {}
local InventoryInsight = InventoryInsight

InventoryInsight.Name = "Vicster's InventoryInsight"
InventoryInsight.Version = "1.6.0"
InventoryInsight.settings = nil
InventoryInsight.data = nil
InventoryInsight.defaultAlertType = UI_ALERT_CATEGORY_ALERT
InventoryInsight.defaultAlertSound = nil
InventoryInsight.WFOLoaded = false
InventoryInsight.RASLoaded = false
InventoryInsight.PlayerLoadedFired = false
InventoryInsight.Integrations = {}

--[[--------------------------------------------------------------
--	Local Variables, Libraries, and Definitions
--]]--------------------------------------------------------------
local LAM = LibStub("LibAddonMenu-1.0")
local LMP = LibStub("LibMediaProvider-1.0")
local LAM_vicstersAddons = _vicstersAddons

local BACKPACK = ZO_PlayerInventoryBackpack
local BANK = ZO_PlayerBankBackpack

local ITEMTOOLTIP = ZO_ItemToolTip
local POPUPTOOLTIP = ZO_PopupToolTip

local CURRENT_ACCOUNT = GetDisplayName()
local CURRENT_CHARACTER = GetUnitName( 'player' )

local IN2_COLORDEF_OBJECT = nil
local IN2_COLORDEF_RED = ZO_ColorDef:New("FF0000")
local IN2_COLORDEF_GREEN = ZO_ColorDef:New("00FF00")
local IN2_COLORDEF_BLUE = ZO_ColorDef:New("0000FF")
local IN2_COLORDEF_DEFAULT = ZO_ColorDef:New("3399FF")

local LEFTPANEL_ORIGINAL_HEIGHT = ZO_SharedThinLeftPanelBackgroundRight:GetHeight()
local LEFTPANEL_NEW_HEIGHT = 1000

--[[--------------------------------------------------------------
--	Global Variables and external functions
--]]--------------------------------------------------------------



IN2_CURRENT_MOUSEOVER_LINK = nil

POPUPTOOLTIP_ORIGINAL_ADDGAMEDATA_HANDLER = PopupTooltip:GetHandler("OnAddGameData");
POPUPTOOLTIP_ORIGINAL_SHOW_HANDLER = PopupTooltip:GetHandler("OnShow");
POPUPTOOLTIP_ORIGINAL_UPDATE_HANDLER = PopupTooltip:GetHandler("OnUpdate");
POPUPTOOLTIP_ORIGINAL_HIDE_HANDLER = PopupTooltip:GetHandler("OnHide");
ITEMTOOLTIP_ORIGINAL_ADDGAMEDATA_HANDLER = ItemTooltip:GetHandler("OnAddGameData");
ITEMTOOLTIP_ORIGINAL_SHOW_HANDLER = ItemTooltip:GetHandler("OnShow");
ITEMTOOLTIP_ORIGINAL_UPDATE_HANDLER = ItemTooltip:GetHandler("OnUpdate");
ITEMTOOLTIP_ORIGINAL_HIDE_HANDLER = ItemTooltip:GetHandler("OnHide");

--[[--------------------------------------------------------------
--	InventoryInsight Prototypes
--]]--------------------------------------------------------------

in2_item = {
	link = "";
	itemInstanceId = "";
	itemCount = 0;
}

in2_bag ={
	items = {};
}

in2_accountCharacter = {
	characterName="";
	bag = in2_bag;
}

in2_inventoryTable = {		
	in2ToggleGuildBankDataCollection = true;
	bankBag = in2_bag;
	guildBanks = {};
	accountCharacters = {};
}

function InventoryInsight.IN2_DissectItemLink( itemLink )
	if(itemLink) then
		local itemColor, data, itemText = itemLink:match("|H(.-):(.-)|h(.-)|h")
		local itemType, 
				itemID, 
				itemQuality, 
				itemLevel, 
				itemEnchantmentType, 
				itemEnchantmentStrength1, 
				itemEnchantmentStrength2,
				_, _, _, _, _, _, _, _, _, 
				itemStyle, 
				_, 
				itemIsBound, 
				itemChargeStatus = zo_strsplit(':', data);

		return itemID, itemText, itemLevel, itemQuality, itemColor, itemStyle, itemType, itemEnchantmentType, itemEnchantmentStrength1, itemEnchantmentStrength2, itemIsBound, itemChargeStatus
	else
		return nil
	end
end

function InventoryInsight.IN2_CompareLinksUsingPlayerPreference( itemLink1, itemLink2 )
	local itemLink1ItemID, itemLink1ItemText, itemLink1ItemLevel, itemLink1ItemValue, itemLink1ItemColor = InventoryInsight.IN2_DissectItemLink(itemLink1);
	local itemLink2ItemID, itemLink2ItemText, itemLink2ItemLevel, itemLink2ItemValue, itemLink2ItemColor = InventoryInsight.IN2_DissectItemLink(itemLink2);
	
	--Set compareResult = using itemID by default. All compares MUST use at least the itemID.
	local compareResult = ( itemLink1ItemID == itemLink2ItemID );

	--[[ ItemLinks are now screwy; removing this code and only comparing by item ID V1.6.0
	--Continue to compare using the player's preference
	if( InventoryInsight.settings.in2CompareByName ) then
		compareResult = compareResult and ( zo_strformat(SI_TOOLTIP_ITEM_NAME, itemLink1ItemText) == zo_strformat(SI_TOOLTIP_ITEM_NAME, itemLink2ItemText) );
		--d(zo_strformat(SI_TOOLTIP_ITEM_NAME, itemLink1ItemText).." "..zo_strformat(SI_TOOLTIP_ITEM_NAME, itemLink2ItemText))
	end
	if( InventoryInsight.settings.in2CompareByLevel ) then
		compareResult = compareResult and ( itemLink1ItemLevel == itemLink2ItemLevel );
	end
	
	if( InventoryInsight.settings.in2CompareByValue ) then
		compareResult = compareResult and ( itemLink1ItemValue == itemLink2ItemValue );
	end
	if( InventoryInsight.settings.in2CompareByColor ) then
		compareResult = compareResult and ( string.upper(itemLink1ItemColor) == string.upper(itemLink2ItemColor) );
	end
		--InventoryInsight.IN2_DebugOutput('[InventoryInsight]:Compare:'..itemLink1..":"..itemLink2);
	]]--	


	return compareResult;
end

--[[--------------------------------------------------------------
--	InventoryInsight.IN2_QueryAccountInventory
		receives: 
			itemLink - string; ZO formatted itemlink
			itemName - string; plain text name of the item
		returns: a queryItem object
		
			queryItem = {
				link = itemLink;
				locations = {
					[n] = {
						name = ""; -- name of the location found; {"Bank","{charactername}"}
						itemsFound = x; -- the number of items of this time found at this location
						worn = false or n; -- where n is the location the slot the item is worn in (currently just itemType)
					}
				}
			}
--]]--------------------------------------------------------------
function InventoryInsight.IN2_QueryAccountInventory( itemLink, itemName )
--function InventoryInsight.IN2_QueryAccountInventoryV2( itemLink, itemName )
	local queryItem = {
		name = itemName;
		link = itemLink;
		locations = {}
	}
	local queryItemsFound = 0;
	for itemName, item in pairs(InventoryInsight.data.DBv2 ) do
		if( queryItem.link ~= nil ) then
			if(InventoryInsight.IN2_CompareLinksUsingPlayerPreference( itemName, queryItem.link )) then
				for locationName, location in pairs(item) do
					if(locationName ~= "attributes") then
						local AlreadySavedLoc = false
						for x, QILocation in pairs(queryItem.locations) do
							if(QILocation.name == locationName)then
								QILocation.itemsFound = QILocation.itemsFound + location.itemCount
								AlreadySavedLoc = true
							end
						end
						if(not AlreadySavedLoc)then
							local newLocation = {};
							newLocation.name = locationName;
							newLocation.itemsFound = location.itemCount;
							newLocation.worn = location.worn;
							table.insert(queryItem.locations, newLocation);
						end
					end		
				end
			end
		elseif( queryItem.name ~= nil ) then
			if( string.upper(itemName):find(string.upper(queryItem.name)) ~= nil ) then
				for locationName, location in pairs(item) do
					if(locationName ~= "attributes") then
						local AlreadySavedLoc = false
						for x, QILocation in pairs(queryItem.locations) do
							if(QILocation.name == locationName)then
								QILocation.itemsFound = QILocation.itemsFound + location.itemCount
								AlreadySavedLoc = true
							end
						end
						if(not AlreadySavedLoc)then
							local newLocation = {};
							newLocation.name = locationName;
							newLocation.itemsFound = location.itemCount;
							newLocation.worn = location.worn;
							table.insert(queryItem.locations, newLocation);
						end
					end		
				end
			end
		end
	end
	return queryItem
end

function InventoryInsight.IN2_TooltipUpdateHandler(control, ...)
	local IN2_queryResults = nil
	local IN2_ProcessedAttribs = false
	local ItemTooltip = ItemTooltip
	local PopupTooltip = PopupTooltip
	local IN2_ITEM_TOOLTIP = IN2_ITEM_TOOLTIP
	local IN2_POPUP_TOOLTIP = IN2_POPUP_TOOLTIP
	if((InventoryInsight.settings.in2ToggleDefaultTooltips and  (control == PopupTooltip or control == ItemTooltip)) or (InventoryInsight.settings.in2ToggleIN2Tooltips and  (control == IN2_POPUP_TOOLTIP or control == IN2_ITEM_TOOLTIP))) then
		if(control == PopupTooltip or control == IN2_POPUP_TOOLTIP) then	--Handle PopupTooltips
			if(not control.IN2_HasModified) then
				InventoryInsight.IN2_DebugOutput('[InventoryInsight]:Updating Tooltip - PopupTooltip');
				IN2_ProcessedAttribs = InventoryInsight.IN2_ProcessItemAttribs(control, control.lastLink);
				IN2_queryResults = InventoryInsight.IN2_QueryAccountInventory(control.lastLink);
				if(#IN2_queryResults.locations > 0) then
					InventoryInsight.IN2_DebugOutput(control.lastLink);
					control:AddLine(" ");
					ZO_Tooltip_AddDivider(control);
					for x, location in pairs(IN2_queryResults.locations) do
						local worn = ""
						if(location.worn and InventoryInsight.settings.in2TooltipsShowWornIndicator)then
							worn = "[Equiped] "
						end
						InventoryInsight.IN2_DebugOutput(location.name.." x"..location.itemsFound);
						control:AddLine(IN2_COLORDEF_OBJECT:Colorize(worn..location.name.." x"..location.itemsFound),LMP:Fetch('font', InventoryInsight.settings.in2TooltipsFont).."|"..InventoryInsight.settings.in2TooltipsFontSize);						
					end	
				else		
					InventoryInsight.IN2_DebugOutput('[InventoryInsight]:No macthing items in DB.');
				end
				control.IN2_HasModified = true;
			end
		elseif(control == ItemTooltip or control == IN2_ITEM_TOOLTIP) then	--Handle ItemTooltips		
			if(not control.IN2_HasModified) then
				InventoryInsight.IN2_DebugOutput('[InventoryInsight]:Updating Tooltip - ItemTooltip');
				IN2_ProcessedAttribs = InventoryInsight.IN2_ProcessItemAttribs(control, IN2_CURRENT_MOUSEOVER_LINK);
				IN2_queryResults = InventoryInsight.IN2_QueryAccountInventory(IN2_CURRENT_MOUSEOVER_LINK);
				if(#IN2_queryResults.locations > 0) then
					InventoryInsight.IN2_DebugOutput(IN2_CURRENT_MOUSEOVER_LINK);
					if(#IN2_queryResults.locations == 1 and InventoryInsight.settings.in2HideRedundantInfo) then
						if(control == IN2_ITEM_TOOLTIP) then
							control:AddLine(" ");
							ZO_Tooltip_AddDivider(control);								
						end
					else
						control:AddLine(" ");
						ZO_Tooltip_AddDivider(control);				
					end
					for x, location in pairs(IN2_queryResults.locations) do
						InventoryInsight.IN2_DebugOutput(location.name.." x"..location.itemsFound);
						local worn = ""
						if(location.worn and InventoryInsight.settings.in2TooltipsShowWornIndicator)then
							worn = "[Equiped] "
						end
						--[[FR105 - v0.1.1 - Adding code]]--
						--[[Change: Adding code to handle InventoryInsight.settings.in2HideRedundantInfo.]]--
						if( not InventoryInsight.settings.in2HideRedundantInfo ) then
							control:AddLine(IN2_COLORDEF_OBJECT:Colorize(worn..location.name.." x"..location.itemsFound),LMP:Fetch('font', InventoryInsight.settings.in2TooltipsFont).."|"..InventoryInsight.settings.in2TooltipsFontSize);
						else
							if( BANK:IsHidden() and InventoryInsight.settings.in2HideRedundantInfo and location.name == CURRENT_CHARACTER ) then
								InventoryInsight.IN2_DebugOutput("[InventoryInsight]:HideRedundantInfo [On] - Skipping last entry.");	
							elseif( BACKPACK:IsHidden() and InventoryInsight.settings.in2HideRedundantInfo and location.name == "Bank") then
								InventoryInsight.IN2_DebugOutput("[InventoryInsight]:HideRedundantInfo [On] - Skipping last entry.");	
							else							
								control:AddLine(IN2_COLORDEF_OBJECT:Colorize(worn..location.name.." x"..location.itemsFound),LMP:Fetch('font', InventoryInsight.settings.in2TooltipsFont).."|"..InventoryInsight.settings.in2TooltipsFontSize);
							end
						end	
						--[[END FR105]]--
					end	
				else		
					InventoryInsight.IN2_DebugOutput('[InventoryInsight]:No macthing items in DB.');
				end
				control.IN2_HasModified = true;
			end
			control.IN2_HasModified = true;
		end
	end
	return IN2_queryResults, IN2_ProcessedAttribs	--Return function for Zgoo use to inspect data query results.
end

function InventoryInsight.IN2_ToolTipShowHandler( control, ... )	
	local ItemTooltip = ItemTooltip
	local PopupTooltip = PopupTooltip
	local IN2_ITEM_TOOLTIP = IN2_ITEM_TOOLTIP
	local IN2_POPUP_TOOLTIP = IN2_POPUP_TOOLTIP
	if(InventoryInsight.settings.in2ToggleDefaultTooltips) then	
		InventoryInsight.IN2_DebugOutput('[InventoryInsight]:Tooltip Detected');
		control.IN2_HasModified = false;
		if(control == ItemTooltip) then
			local mouseOverControl = moc();
			if(mouseOverControl ~= nil) then
	            local mocParent = mouseOverControl:GetParent()
	            if(mocParent)then
	                if(mouseOverControl.dataEntry and mouseOverControl.dataEntry.data.bagId and mouseOverControl.dataEntry.data.slotIndex) then --is inventroy
	                    local bag = mouseOverControl.dataEntry.data.bagId;
	                    local index = mouseOverControl.dataEntry.data.slotIndex;
	                    local curMouseOverLink = GetItemLink(bag, index, LINK_STYLE_BRACKETS);
	                    IN2_CURRENT_MOUSEOVER_LINK = curMouseOverLink;
	                elseif(mouseOverControl:GetParent():GetName() == "ZO_Character") then --is worn item
	                    local bag = mouseOverControl.bagId;
	                    local index = mouseOverControl.itemIndex;
	                    local curMouseOverLink = GetItemLink(bag, index, LINK_STYLE_BRACKETS);
	                    IN2_CURRENT_MOUSEOVER_LINK = curMouseOverLink;
	                elseif(mouseOverControl:GetParent():GetName() == "ZO_StoreWindowListContents") then --is store item
	                    local curMouseOverLink = GetStoreItemLink(mouseOverControl.index, LINK_STYLE_BRACKETS)
	                    IN2_CURRENT_MOUSEOVER_LINK = curMouseOverLink;            
	                elseif(mouseOverControl:GetParent():GetName() == "ZO_BuyBackListContents") then --is store item
	                    local curMouseOverLink = GetBuybackItemLink(mouseOverControl.index, LINK_STYLE_BRACKETS)
	                    IN2_CURRENT_MOUSEOVER_LINK = curMouseOverLink;
	                elseif(mouseOverControl:GetParent():GetName() == "ZO_InteractWindowRewardArea") then --is store item
	                    local curMouseOverLink = GetQuestRewardItemLink(mouseOverControl.index, LINK_STYLE_BRACKETS)
	                    IN2_CURRENT_MOUSEOVER_LINK = curMouseOverLink;	 
	                elseif(mouseOverControl:GetParent():GetName() == "ZO_LootAlphaContainerListContents") then --is loot item
	                    local index = mouseOverControl.dataEntry.data.lootId;
	                    local curMouseOverLink = GetLootItemLink(index, LINK_STYLE_BRACKETS)
	                    IN2_CURRENT_MOUSEOVER_LINK = curMouseOverLink;  	                                       
	                end
	            end
			end
		end
	elseif(InventoryInsight.settings.in2ToggleIN2Tooltips) then
		InventoryInsight.IN2_DebugOutput('[InventoryInsight]:Tooltip Detected');
		control.IN2_HasModified = false;
		if(control == ItemTooltip) then
			IN2_ITEM_TOOLTIP:SetHidden(false)
		elseif(control == PopupTooltip) then
			IN2_POPUP_TOOLTIP:SetHidden(false)
		end	
	end
end

function InventoryInsight.IN2_ToolTipHideHandler( control, ... )
	local ItemTooltip = ItemTooltip
	local PopupTooltip = PopupTooltip
	local IN2_ITEM_TOOLTIP = IN2_ITEM_TOOLTIP
	local IN2_POPUP_TOOLTIP = IN2_POPUP_TOOLTIP
	if(InventoryInsight.settings.in2ToggleDefaultTooltips) then
		InventoryInsight.IN2_DebugOutput('[InventoryInsight]:Tooltip Closed');
		control.IN2_HasModified = false;
		if(control == ItemTooltip) then
			IN2_CURRENT_MOUSEOVER_LINK = nil;
		end	
	elseif(InventoryInsight.settings.in2ToggleIN2Tooltips) then
		InventoryInsight.IN2_DebugOutput('[InventoryInsight]:Tooltip Closed');
		control.IN2_HasModified = false;
		if(control == ItemTooltip) then
			IN2_ITEM_TOOLTIP:SetHidden(true)
		elseif(control == PopupTooltip) then
			IN2_POPUP_TOOLTIP:SetHidden(true)
		end			
	end
end

function InventoryInsight.IN2_ToolTipAddGameDataHandler( control, ... )
	local ItemTooltip = ItemTooltip
	local PopupTooltip = PopupTooltip
	local IN2_ITEM_TOOLTIP = IN2_ITEM_TOOLTIP
	local IN2_POPUP_TOOLTIP = IN2_POPUP_TOOLTIP
	if(InventoryInsight.settings.in2ToggleDefaultTooltips) then
		InventoryInsight.IN2_DebugOutput('[InventoryInsight]:Tooltip Changed');
		control.IN2_HasModified = false;
		local mouseOverControl = moc();
		if(mouseOverControl ~= nil) then
			if(mouseOverControl.dataEntry) then
				local bag = mouseOverControl.dataEntry.data.bagId;
				local index = mouseOverControl.dataEntry.data.slotIndex;
				local curMouseOverLink = GetItemLink(bag, index);
				IN2_CURRENT_MOUSEOVER_LINK = curMouseOverLink;
			elseif( mouseOverControl.bagId and mouseOverControl.itemIndex) then
				local bag = mouseOverControl.bagId;
				local index = mouseOverControl.itemIndex;
				local curMouseOverLink = GetItemLink(bag, index);
				IN2_CURRENT_MOUSEOVER_LINK = curMouseOverLink;
			end
		end
	elseif(InventoryInsight.settings.in2ToggleIN2Tooltips) then
		if(control == ItemTooltip) then
			IN2_ITEM_TOOLTIP:SetHidden(false)
		elseif(control == PopupTooltip) then
			IN2_POPUP_TOOLTIP:SetHidden(false)
		end	
	end
end

function InventoryInsight.IN2_StatusAlert( message )
	ZO_Alert(InventoryInsight.defaultAlertType, InventoryInsight.defaultAlertSound, message)
end

function InventoryInsight.IN2_DebugOutput( output )
	if(InventoryInsight.settings.in2Debug) then
		d(output);
	end
end

function InventoryInsight.IN2_SetDefaultTooltipHandlersV2()
		--PopupTooltips
		ZO_PreHookHandler(PopupTooltip, 'OnShow', InventoryInsight.IN2_ToolTipShowHandler)
		ZO_PreHookHandler(PopupTooltip, 'OnUpdate', InventoryInsight.IN2_TooltipUpdateHandler)
		ZO_PreHookHandler(PopupTooltip, 'OnHide', InventoryInsight.IN2_ToolTipHideHandler)
		ZO_PreHookHandler(PopupTooltip, 'OnAddGameData', InventoryInsight.IN2_ToolTipAddGameDataHandler)
		--ItemTooltips
		ZO_PreHookHandler(ItemTooltip, 'OnShow', InventoryInsight.IN2_ToolTipShowHandler)
		ZO_PreHookHandler(ItemTooltip, 'OnUpdate', InventoryInsight.IN2_TooltipUpdateHandler)
		ZO_PreHookHandler(ItemTooltip, 'OnHide', InventoryInsight.IN2_ToolTipHideHandler)
		ZO_PreHookHandler(ItemTooltip, 'OnAddGameData', InventoryInsight.IN2_ToolTipAddGameDataHandler)
end

function InventoryInsight.InventoryInsight_Loaded(eventCode, addOnName)
	if(addOnName ~= "InventoryInsight") then
        return
    end	
	local defaultSettings = {
		--in2Toggle = true;
		in2ToggleDataCollection = true;
		in2ToggleDefaultTooltips = false;
		in2ToggleIN2Tooltips = true;
		in2Debug = false;
		in2TextColors = IN2_COLORDEF_DEFAULT:ToHex();
		in2HideRedundantInfo = false;
		in2CompareByName = false;
		in2CompareByLevel = false;
		in2CompareByColor = false;
		--in2InventoryFrame options for enabling and dock location
		in2ShowInventoryFrame = true;
		in2ReleaseInventoryFrame = false;
		in2InventoryFrameScenes = { 
			["bank"] = true,
			["guildBank"] = true,
			["tradinghouse"] = true,
			["smithing"] = true,
			["store"] = true,
			["stables"] = true,
			["trade"] = true,
			["inventory"] = true
		},
		in2InventoryFrameSceneSettings = { 
			["bank"] = { locked = false, minimized = false, lastX = 0, lastY = 0},
			["guildBank"] =  { locked = false, minimized = false, lastX = 0, lastY = 0},
			["tradinghouse"] = { locked = false, minimized = false, lastX = 0, lastY = 0},
			["smithing"] = { locked = false, minimized = false, lastX = 0, lastY = 0},
			["store"] = { locked = false, minimized = false, lastX = 0, lastY = 0},
			["stables"] = { locked = false, minimized = false, lastX = 0, lastY = 0},
			["trade"] = { locked = false, minimized = false, lastX = 0, lastY = 0},
			["inventory"] = { locked = false, minimized = false, lastX = 0, lastY = 0}
		},	
		in2DefaultInventoryFrameView = "All",
		in2RelocateOutfitter = true,
		in2RASResearchableTooltipIcon = true,
		in2RASResearchableTooltipIconStyle = "RA 1",
		in2AgedGuildBankDataWarning = true,
		in2InventoryFrameFont = "Univers57",
		in2InventoryFrameFontSize = 16,
		in2TooltipsFont = "Univers57",
		in2TooltipsFontSize = 16,
		in2DefaultTooltipsMinimalPadding = false,
		in2IN2TooltipsMinimalPadding = false,
		in2TooltipsShowWornIndicator = true
	}
	local defaultData = in2_inventoryTable;
	InventoryInsight.settings = ZO_SavedVars:New("InventoryInsight_Settings", 1, nil, defaultSettings)
	InventoryInsight.data = ZO_SavedVars:NewAccountWide("InventoryInsight_Settings", 1, "Data", defaultData)	
	IN2_COLORDEF_OBJECT = ZO_ColorDef:New(InventoryInsight.settings.in2TextColors);	
    SLASH_COMMANDS["/inventoryinsight"] = InventoryInsight.InventoryInsight_SlashCommands;
	InventoryInsight.IN2_RegisterForEvents()
	InventoryInsight.IN2_IsMHOutfitterLoaded() --Handle Outfitter placement if loaded	
	InventoryInsight.IN2_IsResearchAssistantLoaded() --Load RA support if available
	InventoryInsight.IN2_UpdateGuildBankData() --Update account guildBanks (only add, doesn't remove)
	InventoryInsight.IN2_RegisterForSceneChanges() -- register for callbacks on scene statechanges using user preferences or defaults
	InventoryInsight.IN2_CreateSettingsWindow()	
	InventoryInsight.IN2_SetDefaultTooltipHandlersV2();
	InventoryInsight.IN2_SetupIN2InventoryBackpack(InventoryInsight.settings.in2ShowInventoryFrame, InventoryInsight.settings.in2ReleaseInventoryFrame); -- enable/disable inventory frame
	InventoryInsight.IN2_CreateIN2ItemTooltipFrame()
	InventoryInsight.IN2_CreateIN2PopupTooltipFrame()
	--if(InventoryInsight.settings.in2ToggleIN2Tooltips) then	InventoryInsight.IN2_RegisterIN2TooltipsForUpdates(true) end
end

function InventoryInsight.IN2_RegisterForSceneChanges( scene, enable )
	SCENE_MANAGER.scenes.inventory:RegisterCallback("StateChange", function(...)
			InventoryInsight.IN2_ToggleInventoryFrame(SCENE_MANAGER.scenes.inventory, ...)
		end)
	SCENE_MANAGER.scenes.bank:RegisterCallback("StateChange", function(...)
			InventoryInsight.IN2_ToggleInventoryFrame(SCENE_MANAGER.scenes.bank, ...)
		end)
	SCENE_MANAGER.scenes.guildBank:RegisterCallback("StateChange", function(...)
			InventoryInsight.IN2_ToggleInventoryFrame(SCENE_MANAGER.scenes.guildBank, ...)
		end)
	SCENE_MANAGER.scenes.tradinghouse:RegisterCallback("StateChange", function(...)
			InventoryInsight.IN2_ToggleInventoryFrame(SCENE_MANAGER.scenes.tradinghouse, ...)
		end) 
	SCENE_MANAGER.scenes.smithing:RegisterCallback("StateChange", function(...)
			InventoryInsight.IN2_ToggleInventoryFrame(SCENE_MANAGER.scenes.smithing, ...)
		end)
	SCENE_MANAGER.scenes.store:RegisterCallback("StateChange", function(...)
			InventoryInsight.IN2_ToggleInventoryFrame(SCENE_MANAGER.scenes.store, ...)
		end) 
	SCENE_MANAGER.scenes.stables:RegisterCallback("StateChange", function(...)
			InventoryInsight.IN2_ToggleInventoryFrame(SCENE_MANAGER.scenes.stables, ...)
		end) 
	SCENE_MANAGER.scenes.trade:RegisterCallback("StateChange", function(...)
			InventoryInsight.IN2_ToggleInventoryFrame(SCENE_MANAGER.scenes.trade, ...)
		end)
end

function InventoryInsight.IN2_ToggleInventoryFrame(control, oldState, newState)
	InventoryInsight.IN2_DebugOutput("Scene Fired:"..control.name)
	if(InventoryInsight.settings.in2ShowInventoryFrame) then
		if(newState == "showing") then
			if(InventoryInsight.settings.in2InventoryFrameScenes[control.name]) then
				if( not InventoryInsight.settings.in2ReleaseInventoryFrame) then 	--docked
					IN2_INVENTORY_FRAME.ActiveScene = control.name
					if(control.name ~= "inventory" and control.name ~= "smithing") then
						IN2_INVENTORY_FRAME:LoadFrameSceneLocation()
						IN2_INVENTORY_FRAME:SetHidden(false)
						ZO_SharedThinLeftPanelBackground:SetHidden(false)
						ZO_SharedThinLeftPanelBackground:SetWidth(IN2_INVENTORY_FRAME:GetWidth() + 30)
					else
        				ZO_SharedThinLeftPanelBackground:SetWidth(240 + IN2_INVENTORY_FRAME:GetWidth() - 15) 
						IN2_INVENTORY_FRAME:LoadFrameSceneLocation()
						IN2_INVENTORY_FRAME:SetHidden(false)
					end
					ZO_SharedThinLeftPanelBackgroundRight:SetHeight(LEFTPANEL_NEW_HEIGHT)
					ZO_SharedThinLeftPanelBackgroundLeft:SetHeight(LEFTPANEL_NEW_HEIGHT)
				else 	--undocked
					IN2_INVENTORY_FRAME.ActiveScene = control.name
					if(control.name ~= "inventory" and control.name ~= "smithing") then
						IN2_INVENTORY_FRAME:LoadFrameSceneLocation()
						IN2_INVENTORY_FRAME:SetHidden(false)
						ZO_SharedThinLeftPanelBackground:SetHidden(false)
						ZO_SharedThinLeftPanelBackgroundLeft:SetHidden(true)
						ZO_SharedThinLeftPanelBackgroundRight:SetHidden(true)
					else
						IN2_INVENTORY_FRAME:LoadFrameSceneLocation()
						IN2_INVENTORY_FRAME:SetHidden(false)						
						ZO_SharedThinLeftPanelBackground:SetHidden(false)
						ZO_SharedThinLeftPanelBackgroundLeft:SetHidden(false)
						ZO_SharedThinLeftPanelBackgroundRight:SetHidden(false)
					end
				end
			else
				IN2_INVENTORY_FRAME:SetHidden(true)
				if(control.name == "inventory" or control.name == "smithing") then
					ZO_SharedThinLeftPanelBackground:SetWidth(240) 
					ZO_SharedThinLeftPanelBackground:SetHidden(false)
				else
					ZO_SharedThinLeftPanelBackground:SetHidden(true)
					ZO_SharedThinLeftPanelBackgroundLeft:SetHidden(false)
					ZO_SharedThinLeftPanelBackgroundRight:SetHidden(false)
				end
			end
		elseif(newState == "hiding") then
			if(InventoryInsight.settings.in2InventoryFrameScenes[control.name]) then
				if( not InventoryInsight.settings.in2ReleaseInventoryFrame and control.name ~= "inventory" and control.name ~= "smithing") then
					ZO_SharedThinLeftPanelBackground:SetWidth(240 + IN2_INVENTORY_FRAME:GetWidth() - 15) 
					ZO_SharedThinLeftPanelBackground:SetHidden(true)
				elseif(control.name ~= "inventory" and control.name ~= "smithing") then
					IN2_INVENTORY_FRAME:SaveFrameSceneLocation()
					ZO_SharedThinLeftPanelBackground:SetHidden(true)
					ZO_SharedThinLeftPanelBackgroundLeft:SetHidden(false)
					ZO_SharedThinLeftPanelBackgroundRight:SetHidden(false)
				else
					IN2_INVENTORY_FRAME:SaveFrameSceneLocation()
				end
				IN2_INVENTORY_FRAME.ActiveScene = ""
			else
				IN2_INVENTORY_FRAME:SetHidden(true)
				if(control.name == "inventory" or control.name == "smithing") then
					ZO_SharedThinLeftPanelBackground:SetWidth(240) 
					ZO_SharedThinLeftPanelBackground:SetHidden(false)
				else
					ZO_SharedThinLeftPanelBackground:SetHidden(true)
					ZO_SharedThinLeftPanelBackgroundLeft:SetHidden(false)
					ZO_SharedThinLeftPanelBackgroundRight:SetHidden(false)
				end
			end
			ZO_SharedThinLeftPanelBackgroundRight:SetHeight(LEFTPANEL_ORIGINAL_HEIGHT)
			ZO_SharedThinLeftPanelBackgroundLeft:SetHeight(LEFTPANEL_ORIGINAL_HEIGHT)
		end
	end
end

function InventoryInsight.InventoryInsight_SlashCommands( cmd )
	
	if ( cmd == "" ) then
    	d("[InventoryInsight]:Please find the majority of options in the settings section of the menu under Vicster's Addons.")
    	d(" ")
    	d("[InventoryInsight]:Usage - ")
    	d("	/inventoryinsight [options]")
    	d(" 	")
    	d("	Options")
    	d("		debug - Enables debug functionality for the InventoryInsight addon.")
    	d("		run - Runs the InventoryInsight data collector.")
		d("		color - Opens the color picker dialog to set tooltip text color.")
		d("		hideRedundantInfo - Enables/Disables the inventory information added to Tooltips and data collection..")
		return
	end
	
	if ( cmd == "debug" ) then
		if( InventoryInsight.settings.in2Debug ) then
			d("[InventoryInsight]:Debug[Off]")
			InventoryInsight.settings.in2Debug = false;
		else
			d("[InventoryInsight]:Debug[On]")
			InventoryInsight.settings.in2Debug = true;			
		end
		return
	end

	if ( cmd == "run" ) then
		d("[InventoryInsight]:Running collector...")
		InventoryInsight.CollectAll(); 
		return
	end
	
	if ( cmd == "color" ) then
		local in2ColorPickerOnMouseUp = _in2OptionsColorPicker:GetHandler("OnMouseUp");
		in2ColorPickerOnMouseUp( _in2OptionsColorPicker, nil, true);
		return
	end
	
	if ( cmd == "hideRedundantInfo" ) then
		if( InventoryInsight.settings.in2HideRedundantInfo ) then
			d("[InventoryInsight]:HideRedundantInfo[Off]")
			InventoryInsight.settings.in2HideRedundantInfo = false;
		else
			d("[InventoryInsight]:HideRedundantInfo[On]")
			InventoryInsight.settings.in2HideRedundantInfo = true;			
		end
		return
	end
end

function InventoryInsight.IN2_CreateSettingsWindow()

	if( LAM_vicstersAddons == nil) then
		LAM_vicstersAddons = LAM:CreateControlPanel("_vicstersAddons", "Vicster's Addons")
	end

	LAM:AddHeader(LAM_vicstersAddons, "_vicstersAddons_IN2Header", "Inventory Insight")

	LAM:AddDescription(LAM_vicstersAddons, "_in2GeneralSettings", "General configuration settings for Inventory Insight.", "|cc5c29eGeneral Settings|r")

	LAM:AddCheckbox(LAM_vicstersAddons, "_in2OptionsActiveDebugging", 'Debugging', 'Prints verbose debugging to the ChatFrame.',
    function()
      return InventoryInsight.settings.in2Debug;
    end,
    function()
		if( InventoryInsight.settings.in2Debug ) then
			InventoryInsight.IN2_StatusAlert("[InventoryInsight]:Debug[Off]")
			InventoryInsight.settings.in2Debug = false;
		else
			InventoryInsight.IN2_StatusAlert("[InventoryInsight]:Debug[On]")
			InventoryInsight.settings.in2Debug = true;			
		end
		return
    end
	)

	InventoryInsight.CreateSettingsDataCollectionSubWindow( LAM_vicstersAddons )

	InventoryInsight.CreateSettingsDataCleanupSubWindow( LAM_vicstersAddons )

	InventoryInsight.CreateSettingsTooltipOptionsSubWindow( LAM_vicstersAddons )
	--[[ 
	LAM:AddDescription(LAM_vicstersAddons, "_in2OptionsComparePreferenceHeader", "Match items using the following attributes (items will always be compared by ID AND whichever attribute(s) you chose below):", "|cc5c29eItem Comparison Preferences|r")
			
	LAM:AddCheckbox(LAM_vicstersAddons, "_in2OptionsComparePreferenceByName", 'By Name', "Include the item's name in comparisons.",
    function()
      return InventoryInsight.settings.in2CompareByName;
    end,
    function()
		if( InventoryInsight.settings.in2CompareByName ) then
			InventoryInsight.settings.in2CompareByName = false;
		else
			InventoryInsight.settings.in2CompareByName = true;			
		end
		return
    end
	)
	
	LAM:AddCheckbox(LAM_vicstersAddons, "_in2OptionsComparePreferenceByLevel", 'By Level', "Include the item's level in comparisons.",
    function()
      return InventoryInsight.settings.in2CompareByLevel;
    end,
    function()
		if( InventoryInsight.settings.in2CompareByLevel ) then
			InventoryInsight.settings.in2CompareByLevel = false;
		else
			InventoryInsight.settings.in2CompareByLevel = true;			
		end
		return
    end
	)
	

	LAM:AddCheckbox(LAM_vicstersAddons, "_in2ComparePreferenceByValue", 'By Value', "Include the item's value in comparisons.",
    function()
      return InventoryInsight.settings.in2CompareByValue;
    end,
    function()
		if( InventoryInsight.settings.in2CompareByValue ) then
			InventoryInsight.settings.in2CompareByValue = false;
		else
			InventoryInsight.settings.in2CompareByValue = true;			
		end
		return
    end
	)
	
	LAM:AddCheckbox(LAM_vicstersAddons, "_in2OptionsComparePreferenceByColor", 'By Color', "Include the item's color (rarity) in comparisons.",
    function()
      return InventoryInsight.settings.in2CompareByColor;
    end,
    function()
		if( InventoryInsight.settings.in2CompareByColor ) then
			InventoryInsight.settings.in2CompareByColor = false;
		else
			InventoryInsight.settings.in2CompareByColor = true;			
		end
		return
    end
	)

	]]--
	InventoryInsight.CreateSettingsAddonIntegrationSubWindow( LAM_vicstersAddons )

	LAM:AddDescription(LAM_vicstersAddons, "_in2OptionsInventoryFrameSettings", "The InventoryInsight frame provides visual access and searchability to your entire account-wide inventory. The Inventory Frame docks into the Character Frame (left side) and is configurable to be visibile in a range of scenes listed below. You can also choose to undock the frame from the default UI to move it.", "|cc5c29eInventory Frame Settings|r")
	
	LAM:AddCheckbox(LAM_vicstersAddons, "_in2OptionsShowInventoryFrame", 'Show Inventory Frame', "Adds the InventoryInsight Inventory Frame to your default UI. (Visibly from inventory/bank/guild bank/merchant interfaces.)",
    function()
      return InventoryInsight.settings.in2ShowInventoryFrame;
    end,
    function()
		if( InventoryInsight.settings.in2ShowInventoryFrame ) then
			InventoryInsight.IN2_StatusAlert("[InventoryInsight]:ShowInventoryFrame[Off]")
			InventoryInsight.settings.in2ShowInventoryFrame = false;
		else
			InventoryInsight.IN2_StatusAlert("[InventoryInsight]:ShowInventoryFrame[On]")
			InventoryInsight.settings.in2ShowInventoryFrame = true;			
		end
		InventoryInsight.IN2_SetupIN2InventoryBackpack(InventoryInsight.settings.in2ShowInventoryFrame, InventoryInsight.settings.in2ReleaseInventoryFrame);
		return
    end
	)

	LAM:AddCheckbox(LAM_vicstersAddons, "_in2OptionsReleaseInventoryFrame", 'Undock Inventory Frame', "Undocks the InventoryInsight Inventory Frame from your default UI Character Frame (left side). Undocked, the Inventory Frame can be positioned around the screen for the duration of the session.",
    function()
      return InventoryInsight.settings.in2ReleaseInventoryFrame;
    end,
    function()
		if( InventoryInsight.settings.in2ReleaseInventoryFrame ) then
			InventoryInsight.IN2_StatusAlert("[InventoryInsight]:InventoryFrame[Docked]")
			InventoryInsight.settings.in2ReleaseInventoryFrame = false;
		else
			InventoryInsight.IN2_StatusAlert("[InventoryInsight]:InventoryFrame[Undocked]")
			InventoryInsight.settings.in2ReleaseInventoryFrame = true;			
		end
		InventoryInsight.IN2_SetupIN2InventoryBackpack(InventoryInsight.settings.in2ShowInventoryFrame, InventoryInsight.settings.in2ReleaseInventoryFrame);
		return
    end
	)

	LAM:AddDropdown(LAM_vicstersAddons, "_in2OptionsDefaultInventoryFrameView", "Default Inventory Frame View", "The default view (in the dropdown) set when the inventory frame loads.", 
		{ "All", "All Banks", "All Guild Banks", "All Characters", "Bank and Characters", "Bank Only" }, 
	    function()
	      return InventoryInsight.settings.in2DefaultInventoryFrameView;
	    end,
	    function( newValue )
			InventoryInsight.IN2_StatusAlert("[InventoryInsight]:DefaultInventoryFrameView["..newValue.."]")
			InventoryInsight.settings.in2DefaultInventoryFrameView = newValue;
			IN2_INVENTORY_DROPDOWN_CONTROL.dropmenu:SetSelectedItem(newValue)
			InventoryInsight.IN2_SetVisibleInventory(newValue)
			return
	    end
	)

	LAM:AddDropdown(LAM_vicstersAddons, "_in2OptionsInventoryFrameFont", "Inventory Frame Font", "The font used for the items in the Inventory Frame.", 
		LMP:List('font'), 
	    function()
	      return InventoryInsight.settings.in2InventoryFrameFont;
	    end,
	    function( newValue )
			InventoryInsight.IN2_StatusAlert("[InventoryInsight]:InventoryFrameFontChanged["..newValue.."]")
			InventoryInsight.settings.in2InventoryFrameFont = newValue;
			IN2_INVENTORY_FRAME_SCROLL:UpdateFont()
			return
	    end
	)

	LAM:AddDropdown(LAM_vicstersAddons, "_in2OptionsInventoryFrameFontSize", "Inventory Frame Font Size", "The font size used for the items in the Inventory Frame.", 
		{12,13,14,15,16,17,18,19,20}, 
	    function()
	      return InventoryInsight.settings.in2InventoryFrameFontSize;
	    end,
	    function( newValue )
			InventoryInsight.IN2_StatusAlert("[InventoryInsight]:InventoryFrameFontSize["..newValue.."]")
			InventoryInsight.settings.in2InventoryFrameFontSize = newValue;
			IN2_INVENTORY_FRAME_SCROLL:UpdateFont()
			return
	    end
	)	

	if(InventoryInsight.WFOLoaded) then
		LAM:AddCheckbox(LAM_vicstersAddons, "_in2OptionsRelocateOutfitterAddon", 'Relocate Wykkyd\'s Outffiter', "Relocates Wykkyd\'s Outfitter Gear Frame to the top of the Character Apparel Section so the InventoryInsight Inventory Frame does not conflict in location.",
	    function()
	      return InventoryInsight.settings.in2RelocateOutfitter;
	    end,
	    function()
			if( InventoryInsight.settings.in2RelocateOutfitter ) then
				InventoryInsight.IN2_StatusAlert("[InventoryInsight]:RelocateOutfitter[Off]")
				InventoryInsight.settings.in2RelocateOutfitter = false;
			else
				InventoryInsight.IN2_StatusAlert("[InventoryInsight]:RelocateOutfitter[On]")
				InventoryInsight.settings.in2RelocateOutfitter = true;			
			end
			InventoryInsight.IN2_IsMHOutfitterLoaded()
			return
	    end
		)
	end
	LAM:AddDescription(LAM_vicstersAddons, "_in2OptionsInventoryFrameScenes", "Select the scenes below that you want the Inventory Frame to be visible in.", "|cc5c29eAvailable Inventory Frame Scenes|r")

	LAM:AddCheckbox(LAM_vicstersAddons, "_in2OptionsInventoryFrameScene_inventory", 'Inventory scene', "Makes the Inventory Frame visible while viewing your inventory.",
    function()
      return InventoryInsight.settings.in2InventoryFrameScenes["inventory"];
    end,
    function()
		if( InventoryInsight.settings.in2InventoryFrameScenes["inventory"] ) then
			InventoryInsight.IN2_StatusAlert("[InventoryInsight]:InventoryFrameInInventory[Hidden]")
			InventoryInsight.settings.in2InventoryFrameScenes["inventory"] = false;
		else
			InventoryInsight.IN2_StatusAlert("[InventoryInsight]:InventoryFrameInInventory[Visible]")
			InventoryInsight.settings.in2InventoryFrameScenes["inventory"] = true;			
		end
		return
    end
	)

	LAM:AddCheckbox(LAM_vicstersAddons, "_in2OptionsInventoryFrameScene_bank", 'Bank scene', "Makes the Inventory Frame visible while viewing the bank.",
    function()
      return InventoryInsight.settings.in2InventoryFrameScenes["bank"];
    end,
    function()
		if( InventoryInsight.settings.in2InventoryFrameScenes["bank"] ) then
			InventoryInsight.IN2_StatusAlert("[InventoryInsight]:InventoryFrameInBank[Hidden]")
			InventoryInsight.settings.in2InventoryFrameScenes["bank"] = false;
		else
			InventoryInsight.IN2_StatusAlert("[InventoryInsight]:InventoryFrameInBank[Visible]")
			InventoryInsight.settings.in2InventoryFrameScenes["bank"] = true;			
		end
		return
    end
	)

	LAM:AddCheckbox(LAM_vicstersAddons, "_in2OptionsInventoryFrameScene_guildBank", 'Guild Bank Scene', "Makes the Inventory Frame visible while accessing the guild bank.",
    function()
      return InventoryInsight.settings.in2InventoryFrameScenes["guildBank"];
    end,
    function()
		if( InventoryInsight.settings.in2InventoryFrameScenes["guildBank"] ) then
			InventoryInsight.IN2_StatusAlert("[InventoryInsight]:InventoryFrameInGuildBank[Hidden]")
			InventoryInsight.settings.in2InventoryFrameScenes["guildBank"] = false;
		else
			InventoryInsight.IN2_StatusAlert("[InventoryInsight]:InventoryFrameInGuildBank[Visible]")
			InventoryInsight.settings.in2InventoryFrameScenes["guildBank"] = true;			
		end
		return
    end
	)

	LAM:AddCheckbox(LAM_vicstersAddons, "_in2OptionsInventoryFrameScene_tradinghouse", 'Guild Store Scene', "Makes the Inventory Frame visible while accessing the guild store.",
    function()
      return InventoryInsight.settings.in2InventoryFrameScenes["tradinghouse"];
    end,
    function()
		if( InventoryInsight.settings.in2InventoryFrameScenes["tradinghouse"] ) then
			InventoryInsight.IN2_StatusAlert("[InventoryInsight]:InventoryFrameInGuildStore[Hidden]")
			InventoryInsight.settings.in2InventoryFrameScenes["tradinghouse"] = false;
		else
			InventoryInsight.IN2_StatusAlert("[InventoryInsight]:InventoryFrameInGuildStore[Visible]")
			InventoryInsight.settings.in2InventoryFrameScenes["tradinghouse"] = true;			
		end
		return
    end
	)

	LAM:AddCheckbox(LAM_vicstersAddons, "_in2OptionsInventoryFrameScene_smithing", 'Crafting Scene', "Makes the Inventory Frame visible while crafting.",
    function()
      return InventoryInsight.settings.in2InventoryFrameScenes["smithing"];
    end,
    function()
		if( InventoryInsight.settings.in2InventoryFrameScenes["smithing"] ) then
			InventoryInsight.IN2_StatusAlert("[InventoryInsight]:InventoryFrameInCrafting[Hidden]")
			InventoryInsight.settings.in2InventoryFrameScenes["smithing"] = false;
		else
			InventoryInsight.IN2_StatusAlert("[InventoryInsight]:InventoryFrameInCrafting[Visible]")
			InventoryInsight.settings.in2InventoryFrameScenes["smithing"] = true;			
		end
		return
    end
	)

	LAM:AddCheckbox(LAM_vicstersAddons, "_in2OptionsInventoryFrameScene_store", 'Store Scene', "Makes the Inventory Frame visible while buying/selling.",
    function()
      return InventoryInsight.settings.in2InventoryFrameScenes["store"];
    end,
    function()
		if( InventoryInsight.settings.in2InventoryFrameScenes["store"] ) then
			InventoryInsight.IN2_StatusAlert("[InventoryInsight]:InventoryFrameInStore[Hidden]")
			InventoryInsight.settings.in2InventoryFrameScenes["store"] = false;
		else
			InventoryInsight.IN2_StatusAlert("[InventoryInsight]:InventoryFrameInStore[Visible]")
			InventoryInsight.settings.in2InventoryFrameScenes["store"] = true;			
		end
		return
    end
	)

	LAM:AddCheckbox(LAM_vicstersAddons, "_in2OptionsInventoryFrameScene_stables", 'Stables Scene', "Makes the Inventory Frame visible while managing your stables.",
    function()
      return InventoryInsight.settings.in2InventoryFrameScenes["stables"];
    end,
    function()
		if( InventoryInsight.settings.in2InventoryFrameScenes["stables"] ) then
			InventoryInsight.IN2_StatusAlert("[InventoryInsight]:InventoryFrameInStables[Hidden]")
			InventoryInsight.settings.in2InventoryFrameScenes["stables"] = false;
		else
			InventoryInsight.IN2_StatusAlert("[InventoryInsight]:InventoryFrameInStables[Visible]")
			InventoryInsight.settings.in2InventoryFrameScenes["stables"] = true;			
		end
		return
    end
	)

	LAM:AddCheckbox(LAM_vicstersAddons, "_in2OptionsInventoryFrameScene_trade", 'Trade Scene', "Makes the Inventory Frame visible while trading.",
    function()
      return InventoryInsight.settings.in2InventoryFrameScenes["trade"];
    end,
    function()
		if( InventoryInsight.settings.in2InventoryFrameScenes["trade"] ) then
			InventoryInsight.IN2_StatusAlert("[InventoryInsight]:InventoryFrameInTrade[Hidden]")
			InventoryInsight.settings.in2InventoryFrameScenes["trade"] = false;
		else
			InventoryInsight.IN2_StatusAlert("[InventoryInsight]:InventoryFrameInTrade[Visible]")
			InventoryInsight.settings.in2InventoryFrameScenes["trade"] = true;			
		end
		return
    end
	)

	watermark = WINDOW_MANAGER:CreateControl(nil,_vicstersAddons_IN2Header,CT_TEXTURE)
    watermark:SetDimensions(95,95)
    watermark:SetAnchor(TOP, _vicstersAddons_IN2HeaderLabel, BOTTOM, 0, -10)
    watermark:SetTexture("InventoryInsight/assets/inventoryinsight_icon.dds") -- texture will toggle based on current sort
    watermark:SetTextureCoords(0,1,0,1)
    watermark:SetAlpha(1)
end

function InventoryInsight.CreateSettingsDataCleanupSubWindow( panelID )
	local LAM_SubPanel = LAM:AddSubMenu(panelID, "_in2OptionsDataCollectionCleanup", "Manage Collected Data", "Manage collected Characters and Guild Banks. Delete data you no longer need (old guilds or deleted characters).")

	LAM:AddDescription(LAM_SubPanel, "_in2OptionsSubPanelCharactersHeader", "Delete old chracter data that you no longer wish to save by selecting the character's name and pressing delete.", "Collected Character Data")

	

	LAM:AddDropdown(LAM_SubPanel, "_in2OptionsSubPanelCharactersData", "Chracter To Delete", "", 
		InventoryInsight.IN2_CharacterInventories(), 
	    function()
	      	--nothing really to get...
	    end,
	    function()
			--enable delete button
			_in2OptionsSubPanelCharactersDeleteButton.btn:SetEnabled(true)
	    end
	)

	LAM:AddButton(LAM_SubPanel, "_in2OptionsSubPanelCharactersDeleteButton", "Delete Character", "Delete Inventory Insight data for the character selected above.", 
		function()
			local selectedCharacter = _in2OptionsSubPanelCharactersDataDropdown.m_comboBox:GetSelectedItem()
			if(selectedCharacter ~= "") then
				--delete selected character
			    for x, character in pairs(InventoryInsight.data.accountCharacters) do
			        if(character.characterName == selectedCharacter) then
			        	table.remove(InventoryInsight.data.accountCharacters, x)
			        end
			    end 
				--remove item from dropdown
				_in2OptionsSubPanelCharactersDataDropdown.m_comboBox:ClearItems()
				local CharacterInventories = InventoryInsight.IN2_CharacterInventories()
			    for i=1,#CharacterInventories do  
			        local entry = _in2OptionsSubPanelCharactersDataDropdown.m_comboBox:CreateItemEntry(CharacterInventories[i], nil)  
			       _in2OptionsSubPanelCharactersDataDropdown.m_comboBox:AddItem(entry) 
			    end
			end
			--disable button
			_in2OptionsSubPanelCharactersDeleteButton.btn:SetEnabled(false)
		end, 
		true, 
		"All data for the selected character above will be deleted!"
	)
	_in2OptionsSubPanelCharactersDeleteButton.btn:SetEnabled(false)

	LAM:AddDescription(LAM_SubPanel, "_in2OptionsSubPanelGuildBanksHeader", "Delete old guild bank data that you no longer wish to save by selecting the guild's name and pressing delete.", "Collected Guild Bank Data")

	LAM:AddDropdown(LAM_SubPanel, "_in2OptionsSubPanelGuildBanksData", "Guild Bank To Delete", "", 
		InventoryInsight.IN2_GuildBanks(), 
	    function()
	      	--
	    end,
	    function()
			--enable delete button
			_in2OptionsSubPanelGuildBankDeleteButton.btn:SetEnabled(true)
	    end
	)

	LAM:AddButton(LAM_SubPanel, "_in2OptionsSubPanelGuildBankDeleteButton", "Delete Guild Bank", "Delete Inventory Insight data for the guild selected above.", 
		function()
			local selectedGuild = _in2OptionsSubPanelGuildBanksDataDropdown.m_comboBox:GetSelectedItem()
			if(selectedGuild ~= "") then
				--delete selected character
				InventoryInsight.data.guildBanks[selectedGuild] = nil
				--remove item from dropdown
				_in2OptionsSubPanelGuildBanksDataDropdown.m_comboBox:ClearItems()
				local GuildBanks = InventoryInsight.IN2_GuildBanks()
			    for i=1,#GuildBanks do  
			        local entry = _in2OptionsSubPanelGuildBanksDataDropdown.m_comboBox:CreateItemEntry(GuildBanks[i], nil)  
			       _in2OptionsSubPanelGuildBanksDataDropdown.m_comboBox:AddItem(entry) 
			    end
			end
			--disable button
			_in2OptionsSubPanelGuildBankDeleteButton.btn:SetEnabled(false)
		end, 
		true, 
		"All data for the selected guild bank above will be deleted!"
	)
	_in2OptionsSubPanelGuildBankDeleteButton.btn:SetEnabled(false)
end

function InventoryInsight.CreateSettingsAddonIntegrationSubWindow( panelID )
	if(InventoryInsight.RASLoaded)then
		local LAM_SubPanel = LAM:AddSubMenu(panelID, "_in2OptionsAddonIntegrations", "External Addon Integrations", "Options for integrations with external addons that Inventory Insight supports.")

		LAM:AddDescription(LAM_SubPanel, "_in2OptionsSubPanelRAIntegration", "Integration options with the addon \"Research Assistant\" by Ingeniousclown.", "Research Assistant Integration")

		LAM:AddCheckbox(LAM_SubPanel, "_in2OptionsSubPanelRAIntegrationResearchable", 'Researchable Tooltip Icon', "Enabling this feature will add an icon to the custom IN2 ItemTooltip when an item is researchable.",
	    function()
	      return InventoryInsight.settings.in2RASResearchableTooltipIcon;
	    end,
	    function()
			if( InventoryInsight.settings.in2RASResearchableTooltipIcon ) then
				InventoryInsight.IN2_StatusAlert("[InventoryInsight]:ResearchAssistantIntegration[Off]")
				InventoryInsight.settings.in2RASResearchableTooltipIcon = false;
			else
				InventoryInsight.IN2_StatusAlert("[InventoryInsight]:ResearchAssistantIntegration[On]")
				InventoryInsight.settings.in2RASResearchableTooltipIcon = true;			
			end
			return
	    end
		)

		LAM:AddDropdown(LAM_SubPanel, "_in2OptionsSubPanelRAIntegrationResearchableIcons", "RA Tooltip Icon", "The icon used to represent researchable items as shown in the custom IN2 tooltips.", 
			{"RA 1","RA 2","RA 3","RA 4","RA 5","RA 6","RA 7","RA 8","RA 9","RA 10"}, 
		    function()
		      return InventoryInsight.settings.in2RASResearchableTooltipIconStyle;
		    end,
		    function( newValue )
				InventoryInsight.IN2_StatusAlert("[InventoryInsight]:TooltipsFontSizeChanged["..newValue.."]")
				InventoryInsight.settings.in2RASResearchableTooltipIconStyle = newValue;
				IN2_ITEM_TOOLTIP.RAIntegration:SetTexture(InventoryInsight.Integrations.RATooltipIcons[InventoryInsight.settings.in2RASResearchableTooltipIconStyle])
				_in2OptionsSubPanelRAIntegrationResearchableIcons.RAIntegrationOptionsIcon:SetTexture(InventoryInsight.Integrations.RATooltipIcons[InventoryInsight.settings.in2RASResearchableTooltipIconStyle])
				return
		    end
		)		

		_in2OptionsSubPanelRAIntegrationResearchableIcons.RAIntegrationOptionsIcon = WINDOW_MANAGER:CreateControl("RAIntegrationOptionsIcon", LAM_SubPanel, CT_TEXTURE)
		local RAIntegrationOptionsIcon = _in2OptionsSubPanelRAIntegrationResearchableIcons.RAIntegrationOptionsIcon
	    RAIntegrationOptionsIcon:SetDimensions(25,25)
	    RAIntegrationOptionsIcon:SetAnchor(RIGHT, _in2OptionsSubPanelRAIntegrationResearchableIcons, RIGHT, -200, 0)
	    RAIntegrationOptionsIcon:SetTexture(InventoryInsight.Integrations.RATooltipIcons[InventoryInsight.settings.in2RASResearchableTooltipIconStyle])
	    RAIntegrationOptionsIcon:SetTextureCoords(0,1,0,1)
	    RAIntegrationOptionsIcon:SetColor(1, .25, 0, 1)
	    RAIntegrationOptionsIcon:SetAlpha(1)

	end
end

function InventoryInsight.CreateSettingsDataCollectionSubWindow( panelID )
	local LAM_SubPanel = LAM:AddSubMenu(panelID, "_in2OptionsDataCollectionSubWindow", "Data Collection", "Managae data collection options for inventories and guild banks.")

	LAM:AddCheckbox(LAM_SubPanel, "_in2OptionsDataCollection", 'Data Collection', 'Enables/Disables data collection for this character.',
    function()
      return InventoryInsight.settings.in2ToggleDataCollection;
    end,
    function()
        if InventoryInsight.settings.in2ToggleDataCollection then
    		InventoryInsight.settings.in2ToggleDataCollection = false;
    		InventoryInsight.IN2_StatusAlert("[InventoryInsight]:Data Collection[disabled]")		
			--InventoryInsight.IN2_SetDefaultTooltipHandlers(false); -- turn of handlers
    	else
		   	InventoryInsight.settings.in2ToggleDataCollection = true;
    		InventoryInsight.IN2_StatusAlert("[InventoryInsight]:Data Collection[enabled]")
			InventoryInsight.CollectAll();		
			--InventoryInsight.IN2_SetDefaultTooltipHandlers(true); -- turn on handlers
    	end
    end,
    true,
    "Inventory information will not be updated if this option is turned off!"
	)

	LAM:AddCheckbox(LAM_SubPanel, "_in2OptionsGuildBankDataCollection", 'Guild Bank Data Collection', 'Enables/Disables data collection for guild banks on this account.',
    function()
      return InventoryInsight.data.in2ToggleGuildBankDataCollection;
    end,
    function()
        if InventoryInsight.data.in2ToggleGuildBankDataCollection then
    		InventoryInsight.data.in2ToggleGuildBankDataCollection = false;
    		InventoryInsight.IN2_StatusAlert("[InventoryInsight]:Guild Bank Data Collection[disabled]")	
    	else
		   	InventoryInsight.data.in2ToggleGuildBankDataCollection = true;
    		InventoryInsight.IN2_StatusAlert("[InventoryInsight]:Guild Bank Data Collection[enabled]")
    	end
    end,
    true,
    "Guild bank information will not be updated if this option is turned off!"
	)

	LAM:AddCheckbox(LAM_SubPanel, "_in2OptionsAgedGuildBankDataWarningWarning", 'Old Guild Bank Data Alert', 'Enables/Disables an alert that will notify you once, when you first log in, if one or more guild banks contain data that is 5 days or older.',
    function()
      return InventoryInsight.data.in2AgedGuildBankDataWarning;
    end,
    function()
        if InventoryInsight.data.in2AgedGuildBankDataWarning then
    		InventoryInsight.data.in2AgedGuildBankDataWarning = false;
    		InventoryInsight.IN2_StatusAlert("[InventoryInsight]:Aged Guild Bank Data Warning[disabled]")	
    	else
		   	InventoryInsight.data.in2AgedGuildBankDataWarning = true;
    		InventoryInsight.IN2_StatusAlert("[InventoryInsight]:Aged Guild Bank Data Warning[enabled]")
    	end
    end
	)
end

function InventoryInsight.CreateSettingsTooltipOptionsSubWindow( panelID )
	local LAM_SubPanel = LAM:AddSubMenu(panelID, "_in2OptionsTooltipOptionsSubWindow", "Tooltips", "Managae tooltip options for both default and custom IN2 tooltips.")

	LAM:AddCheckbox(LAM_SubPanel, "_in2OptionsDefaultTooltips", 'Use Default Tooltips', 'Enables/Disables inventory insight data added to the default tooltips. (Turning this feature on will disable the InventoryInsight Tooltips control.)',
    function()
      return InventoryInsight.settings.in2ToggleDefaultTooltips;
    end,
    function()
        if InventoryInsight.settings.in2ToggleDefaultTooltips then
    		InventoryInsight.settings.in2ToggleDefaultTooltips = false;
    		InventoryInsight.IN2_StatusAlert("[InventoryInsight]:Default UI Tooltips[disabled]")		
			--no longer turning off handlers as this can cause problems with other addons!! with this disabled, the tooltips will no longer show.
    	else
		   	InventoryInsight.settings.in2ToggleDefaultTooltips = true;
    		InventoryInsight.IN2_StatusAlert("[InventoryInsight]:Default UI Tooltips[enabled]")
		   	if(InventoryInsight.settings.in2ToggleIN2Tooltips) then 
		   		InventoryInsight.settings.in2ToggleIN2Tooltips = false
		   		_in2OptionsIN2Tooltips:GetNamedChild("Checkbox"):toggleFunction(InventoryInsight.in2ToggleIN2Tooltips)
    			InventoryInsight.IN2_StatusAlert("[InventoryInsight]:InventoryInsight Tooltips[disabled]")
		   	end
    	end
    end
	)

	LAM:AddCheckbox(LAM_SubPanel, "_in2OptionsIN2Tooltips", 'Use InventoryInsight Tooltips', 'Enables/Disables inventory insight data added to the InventoryInsight tooltips. (Turning this feature on will disable adding data to the default Tooltips control.)',
    function()
      return InventoryInsight.settings.in2ToggleIN2Tooltips;
    end,
    function()
        if InventoryInsight.settings.in2ToggleIN2Tooltips then
    		InventoryInsight.settings.in2ToggleIN2Tooltips = false;
    		InventoryInsight.IN2_StatusAlert("[InventoryInsight]:InventoryInsight Tooltips[disabled]")		
			--no longer turning off handlers as this can cause problems with other addons!! with this disabled, the tooltips will no longer show.
    	else
		   	InventoryInsight.settings.in2ToggleIN2Tooltips = true;
    		InventoryInsight.IN2_StatusAlert("[InventoryInsight]:InventoryInsight Tooltips[enabled]")
		   	if(InventoryInsight.settings.in2ToggleDefaultTooltips) then 
		   		InventoryInsight.settings.in2ToggleDefaultTooltips = false
		   		_in2OptionsDefaultTooltips:GetNamedChild("Checkbox"):toggleFunction(InventoryInsight.in2ToggleDefaultTooltips)
    			InventoryInsight.IN2_StatusAlert("[InventoryInsight]:Default UI Tooltips[disabled]")
		   	end
    	end
    end
	)
  
	--[[FR105 - v0.1.1 - Adding code]]--
	--[[Change: Adding code to handle the InventoryInsight.settings.in2HideRedundantInfo option in the InventoryInsight.settings panel.]]--
    LAM:AddCheckbox(LAM_SubPanel, "_in2OptionsHideRedundantInfo", 'Hide Redundant Item Information', 'Option to enable/disable showing information on ItemTooltips (inventory and bank) when already viewing that bag. (i.e., If you mouse over an item in your inventory you can already see how many you have.)',
    function()
      return InventoryInsight.settings.in2HideRedundantInfo;
    end,
    function()
		if( InventoryInsight.settings.in2HideRedundantInfo ) then
			InventoryInsight.IN2_StatusAlert("[InventoryInsight]:HideRedundantInfo[Off]")
			InventoryInsight.settings.in2HideRedundantInfo = false;
		else
			InventoryInsight.IN2_StatusAlert("[InventoryInsight]:HideRedundantInfo[On]")
			InventoryInsight.settings.in2HideRedundantInfo = true;			
		end
		return
    end
	)
	--[[END FR105]]--

    LAM:AddCheckbox(LAM_SubPanel, "_in2OptionsShowWornIndicator", 'Show Worn Indicators', 'Option to enable/disable displaying \'[WORN]\' in front of location if the item in the tooltip is a worn piece of equipment on that character.',
    function()
      return InventoryInsight.settings.in2TooltipsShowWornIndicator;
    end,
    function()
		if( InventoryInsight.settings.in2TooltipsShowWornIndicator ) then
			InventoryInsight.IN2_StatusAlert("[InventoryInsight]:ShowWornIndicators[Off]")
			InventoryInsight.settings.in2TooltipsShowWornIndicator = false;
		else
			InventoryInsight.IN2_StatusAlert("[InventoryInsight]:ShowWornIndicators[On]")
			InventoryInsight.settings.in2TooltipsShowWornIndicator = true;			
		end
		return
    end
	)

    LAM:AddColorPicker(LAM_SubPanel, "_in2OptionsColorPicker", 'Tooltip Inventory Information Text Color', 'Sets the color of the text for the inventory information that gets added to Tooltips.',
    function()
      return IN2_COLORDEF_OBJECT:UnpackRGBA();
    end,
    function(...)
		IN2_COLORDEF_OBJECT:SetRGBA(...);
		InventoryInsight.settings.in2TextColors = IN2_COLORDEF_OBJECT:ToHex();
		return
    end
    )

	LAM:AddDropdown(LAM_SubPanel, "_in2OptionsTooltipsFont", "Tooltips Font", "The font used for location information added to both default and custom IN2 tooltips.", 
		LMP:List('font'), 
	    function()
	      return InventoryInsight.settings.in2TooltipsFont;
	    end,
	    function( newValue )
			InventoryInsight.IN2_StatusAlert("[InventoryInsight]:TooltipsFontChanged["..newValue.."]")
			InventoryInsight.settings.in2TooltipsFont = newValue;
			return
	    end
	)

	LAM:AddDropdown(LAM_SubPanel, "_in2OptionsTooltipsFontSize", "Tooltips Font Size", "The font size used for location information added to both default and custom IN2 tooltips.", 
		{12,13,14,15,16,17,18,19,20}, 
	    function()
	      return InventoryInsight.settings.in2TooltipsFontSize;
	    end,
	    function( newValue )
			InventoryInsight.IN2_StatusAlert("[InventoryInsight]:TooltipsFontSizeChanged["..newValue.."]")
			InventoryInsight.settings.in2TooltipsFontSize = newValue;
			return
	    end
	)

	

    LAM:AddCheckbox(LAM_SubPanel, "_in2OptionsDefaultTooltipsMinimalPadding", 'Minimal Padding Default Tooltips', 'Option to enable/disable minimal padding inside the default ItemTooltips and PopupTooltips in the UI.',
    function()
      return InventoryInsight.settings.in2DefaultTooltipsMinimalPadding;
    end,
    function()
    	local ItemTooltip = ItemTooltip
    	local PopupTooltip = PopupTooltip
		if( InventoryInsight.settings.in2DefaultTooltipsMinimalPadding ) then
			InventoryInsight.IN2_StatusAlert("[InventoryInsight]:DefaultTooltipsMinimalPadding[Off]")
			InventoryInsight.settings.in2DefaultTooltipsMinimalPadding = false;
			InventoryInsight.IN2_AdjustTooltipPadding(ItemTooltip, false)
			InventoryInsight.IN2_AdjustTooltipPadding(PopupTooltip, false)
		else
			InventoryInsight.IN2_StatusAlert("[InventoryInsight]:DefaultTooltipsMinimalPadding[On]")
			InventoryInsight.settings.in2DefaultTooltipsMinimalPadding = true;
			InventoryInsight.IN2_AdjustTooltipPadding(ItemTooltip, true)
			InventoryInsight.IN2_AdjustTooltipPadding(PopupTooltip, true)	
		end
		return
    end,
    true,
    "This affects the default tooltips even when not being used by Inventory Insight!"
	)

    LAM:AddCheckbox(LAM_SubPanel, "_in2OptionsIN2TooltipsMinimalPadding", 'Minimal Padding IN2 Tooltips', 'Option to enable/disable minimal padding inside the Inventory Insight ItemTooltips and PopupTooltips in the UI.',
    function()
      return InventoryInsight.settings.in2IN2TooltipsMinimalPadding;
    end,
    function()
    	local ItemTooltip = IN2_ITEM_TOOLTIP
    	local PopupTooltip = IN2_POPUP_TOOLTIP
		if( InventoryInsight.settings.in2IN2TooltipsMinimalPadding ) then
			InventoryInsight.IN2_StatusAlert("[InventoryInsight]:IN2TooltipsMinimalPadding[Off]")
			InventoryInsight.settings.in2IN2TooltipsMinimalPadding = false;
			InventoryInsight.IN2_AdjustTooltipPadding(ItemTooltip, false)
			InventoryInsight.IN2_AdjustTooltipPadding(PopupTooltip, false)
		else
			InventoryInsight.IN2_StatusAlert("[InventoryInsight]:IN2TooltipsMinimalPadding[On]")
			InventoryInsight.settings.in2IN2TooltipsMinimalPadding = true;		
			InventoryInsight.IN2_AdjustTooltipPadding(ItemTooltip, true)
			InventoryInsight.IN2_AdjustTooltipPadding(PopupTooltip, true)		
		end
		return
    end
	)
	
end

function InventoryInsight.InventoryInsight_Initialized()
	EVENT_MANAGER:RegisterForEvent("InventoryInsightLoaded", EVENT_ADD_ON_LOADED, InventoryInsight.InventoryInsight_Loaded)
end


InventoryInsight.InventoryInsight_Initialized()
